/***********************************************************************
 * File: byteorder_test.c
 * 	(Extracted from
 *	   "$Id: byteorder_test.c,v 1.6 2009/06/05 19:24:44 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source file implements test program for `byteorder' API-s.
 *
 * Copyright (C) 2007-2009 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include "byteorder.h"

int main(int ac, char **av)
{
	struct val_t {
		char	a_code;
		short	a_word;
		int32_t	a_long;
		#define N_codes 4
		char	codes[N_codes];
		#define N_words 3
		short	words[N_words];
		#define N_longs 5
		int32_t	longs[N_longs];
	}					val =	{ '#'
								, 0x100
								, 0x1020300
								, "%@!"
								, { 0x102, 0x304, 0x500 }
								, { 0x1020304, 0x05060708, 0x90a0b0c
									, 0xd0e0f01, 0x2030400
									}
								};
	struct net_t {
		char	a_code;
		char	a_word[sizeof(short)];
		char	a_long[sizeof(int32_t)];
		char	codes[N_codes];
		char	words[N_words * sizeof(short)];
		char	longs[N_longs * sizeof(int32_t)];
	}					rev_host
						, net = {
						  '#'
						, "\x1" /* 256 + 0 = 256 */
						, "\x1\x2\x3" /* = 16909056 */
						, "%@!"
						, "\x1\x2" "\x3\x4" "\x5" /* = [256, 512, 768] */
						, "\x1\x2\x3\x4" "\x5\x6\x7\x8" "\x9\xa\xb\xc"
						  "\xd\xe\xf\x1" "\x2\x3\x4"
							/* = [16909056, 84281088, 151653120,
							 *	219025152, 33752064]
							 */
						};
	struct host_t {
		char	a_code;
		short	a_word;
		int32_t	a_long;
		char	codes[N_codes];
		short	words[N_words];
		int32_t	longs[N_longs];
	}					host;
	Byteorder_layout_t	layouts[] = { { Byteorder_type_e_char, 1
										, offsetof(struct net_t, a_code)
										, offsetof(struct host_t, a_code)
										}
									, { Byteorder_type_e_short, 1
										, offsetof(struct net_t, a_word)
										, offsetof(struct host_t, a_word)
										}
									, { Byteorder_type_e_long, 1
										, offsetof(struct net_t, a_long)
										, offsetof(struct host_t, a_long)
										}
									, { Byteorder_type_e_char, N_codes
										, offsetof(struct net_t, codes)
										, offsetof(struct host_t, codes)
										}
									, { Byteorder_type_e_short, N_words
										, offsetof(struct net_t, words)
										, offsetof(struct host_t, words)
										}
									, { Byteorder_type_e_long, N_longs
										, offsetof(struct net_t, longs)
										, offsetof(struct host_t, longs)
										}
									};
	size_t				gprof_cnt, gprof_iter_max;
	char				net_host[11], *val_str = "12345"
						, *result, *result1, *result2
						, **sv_av =av
						;
	int					sv_errno, sv_ac = ac, exit_code = 1;

	for (gprof_iter_max = (size_t)((ac == 2 && strcmp(av[1], "-P") == 0)?
																10000: 1
									)
		, gprof_cnt = 0
	; gprof_cnt < gprof_iter_max
	; ++gprof_cnt
	) {	/* iterate the unit test sequence for gprof profiling... */

	ac = sv_ac; av = sv_av
	; printf("{{{ byteorder_test starts...\n"); (void)fflush(stdout)
	; memset(&host, (int)'\0', sizeof(host))
	;
	if (((strcpy((char *)memset(net_host, (int)'\0', sizeof(net_host))
								, val_str
								) != net_host)
	|| (result = byteorder_chars( Byteorder_dir_e_ntoh
								, net_host
								, (net_host + (strlen(val_str) / 2))
								, strlen(net_host)
								)) == NULL)
	|| strcmp(val_str, (net_host + (strlen(val_str) / 2))) != 0
	) {
		sv_errno	= errno;

		printf(	"!!!FATAL!!!:L%d: failed in byteorder_chars( "
				"overlapped ) (errno=%d[%s], net_host=[%s]).\n"
				, __LINE__, sv_errno, strerror(sv_errno), net_host
				);
		; (void)fflush(stdout)
		;
	}
	else if (((strcpy((char *)memset(net_host, (int)'\0', sizeof(net_host))
									, val_str
									) != net_host)
	|| (result = byteorder_chars( Byteorder_dir_e_hton
								, (net_host + (strlen(val_str) / 2))
								, net_host
								, strlen(net_host)
								)) == NULL)
	|| strcmp(val_str, (net_host + (strlen(val_str) / 2))) != 0
	) {
		sv_errno	= errno;

		printf(	"!!!FATAL!!!:L%d: failed in byteorder_chars( "
				"overlapped ) (errno=%d[%s], net_host=[%s]).\n"
				, __LINE__
				, sv_errno, strerror(sv_errno), net_host
				);
		; (void)fflush(stdout)
		;
	}
	else if ((result1 = byteorder(Byteorder_dir_e_ntoh
								, (char *)(&net)
								, sizeof(net)
								, (sizeof(layouts) / sizeof(layouts[0]))
								, layouts
								, (char *)(&host)
								, sizeof(host)
								)) == NULL
	) {
		sv_errno	= errno;

		printf(	"!!!FATAL!!!:L%d: Byteorder_dir_e_ntoh "
				"(errno=%d[%s].\n"
				, __LINE__, sv_errno, strerror(sv_errno)
				)
		; (void)fflush(stdout)
		;
	}
	else if (memcmp(&(host.a_code), &(val.a_code), sizeof(host.a_code)) != 0
	) {
		printf(	"!!!FATAL!!!:L%d: host.a_code=(%d:0x%x), "
				"val.a_code=(%d:0x%x).\n"
				, __LINE__
				, (int)host.a_code, (unsigned)host.a_code
				, (int)val.a_code, (unsigned)val.a_code
				)
		; (void)fflush(stdout)
		;
	}
	else if (memcmp(&(host.a_word), &(val.a_word), sizeof(host.a_word)) != 0
	) {
		printf(	"!!!FATAL!!!:L%d: host.a_word=(%d:0x%x), "
				"val.a_word=(%d:0x%x).\n"
				, __LINE__
				, host.a_word, (unsigned)host.a_word
				, val.a_word, (unsigned)val.a_word
				)
		; (void)fflush(stdout)
		;
	}
	else if (memcmp(&(host.a_long), &(val.a_long), sizeof(host.a_long)) != 0
	) {
		printf(	"!!!FATAL!!!:L%d: host.a_long=(%ld:0x%lx), "
				"val.a_long=(%ld:0x%lx).\n"
				, __LINE__
				, (long)host.a_long,	(unsigned long)host.a_long
				, (long)val.a_long,		(unsigned long)val.a_long
				)
		; (void)fflush(stdout)
		;
	}
	else if (memcmp(host.codes, val.codes, sizeof(host.codes)) != 0) {
		printf(	"!!!FATAL!!!:L%d: `codes' members of host and "
				"val are differ.\n"
				, __LINE__
				)
		; (void)fflush(stdout)
		;
	}
	else if (memcmp(host.words, val.words, sizeof(host.words)) != 0) {
		printf(	"!!!FATAL!!!:L%d: `words' members of host and "
				"val are differ.\n"
				, __LINE__
				)
		; (void)fflush(stdout)
		;
	}
	else if (memcmp(host.longs, val.longs, sizeof(host.longs)) != 0) {
		printf(	"!!!FATAL!!!:L%d: `longs' members of host and "
				"val are differ.\n"
				, __LINE__
				)
		; (void)fflush(stdout)
		;
	}
	else if ((result2 = byteorder(Byteorder_dir_e_hton
								, (char *)(&rev_host)
								, sizeof(rev_host)
								, (sizeof(layouts) / sizeof(layouts[0]))
								, layouts
								, (char *)(&host)
								, sizeof(host)
								)) == NULL
	) {
		sv_errno	= errno;
		printf(	"!!!FATAL!!!:L%d: Byteorder_dir_e_hton (errno=%d[%s]).\n"
				, __LINE__, sv_errno, strerror(sv_errno)
				)
		; (void)fflush(stdout)
		;
	}
	else if (memcmp(&net, &rev_host, sizeof(net)) != 0) {
		printf(	"!!!FATAL!!!:L%d: 'rev_host' is differ than `net'.\n"
				, __LINE__
				)
		; (void)fflush(stdout)
		;
	}
	else {
		exit_code	= 0; /* exit normally */
	}

	printf("}}} byteorder_test finished!\n"); (void)fflush(stdout);

	} /* iterate the unit test sequence for gprof profiling... */

	exit(exit_code);
}
